@@ -14,11 +14,11 @@ import copy
 import re
 import shlex
 
-from ironic_lib import disk_utils
 from ironic_lib import utils as il_utils
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 
+from ironic_python_agent import disk_utils
 from ironic_python_agent import errors
 from ironic_python_agent import utils
 
@@ -227,7 +227,7 @@ def create_raid_device(index, logical_disk):
     volume_name = logical_disk.get('volume_name')
     try:
         if volume_name is None:
-            volume_name = 'md%d' % index
+            volume_name = md_device
         LOG.debug("Creating md device %(dev)s with name %(name)s"
                   "on %(comp)s",
                   {'dev': md_device, 'name': volume_name,
