@@ -22,7 +22,7 @@ from ironic_python_agent.tests.unit import base
 
 
 @mock.patch('ironic_lib.utils.mounted', autospec=True)
-@mock.patch('ironic_lib.disk_utils.list_partitions', autospec=True)
+@mock.patch('ironic_python_agent.disk_utils.list_partitions', autospec=True)
 @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
             lambda _call: '/dev/fake')
 class TestFindPartitionWithPath(base.IronicAgentTest):
@@ -228,7 +228,7 @@ class TestInjectOne(base.IronicAgentTest):
         with open(self.path, 'rb') as fp:
             self.assertEqual(b'content', fp.read())
         self.assertEqual(0o602, stat.S_IMODE(os.stat(self.path).st_mode))
-        # Exising directories do not change their permissions
+        # Existing directories do not change their permissions
         self.assertNotEqual(0o703, stat.S_IMODE(os.stat(self.dirpath).st_mode))
 
         mock_find_and_mount.assert_called_once_with(fl['path'], None,
