@@ -23,6 +23,7 @@ import json
 import os
 import re
 import shutil
+import stat
 import subprocess
 import sys
 import tarfile
@@ -59,7 +60,7 @@ AGENT_PARAMS_CACHED = dict()
 
 
 LSBLK_COLUMNS = ['KNAME', 'MODEL', 'SIZE', 'ROTA',
-                 'TYPE', 'UUID', 'PARTUUID', 'SERIAL']
+                 'TYPE', 'UUID', 'PARTUUID', 'SERIAL', 'WWN']
 
 
 DEVICE_EXTRACTOR = re.compile(r'^(?:(.*\d)p|(.*\D))(?:\d+)$')
@@ -224,7 +225,7 @@ def _find_mount_point(device):
 def _check_vmedia_device(vmedia_device_file):
     """Check if a virtual media device appears valid.
 
-    Explicitly ignores partitions, actual disks, and other itmes that
+    Explicitly ignores partitions, actual disks, and other items that
     seem unlikely to be virtual media based items being provided
     into the running operating system via a BMC.
 
@@ -311,18 +312,27 @@ def copy_config_from_vmedia():
         ['config-2', 'vmedia_boot_iso'])
     if not vmedia_device_file:
         _early_log('No virtual media device detected')
+        _unmount_any_config_drives()
         return
     if not _booted_from_vmedia():
         _early_log('Cannot use configuration from virtual media as the '
                    'agent was not booted from virtual media.')
+        _unmount_any_config_drives()
         return
     # Determine the device
     mounted = _find_mount_point(vmedia_device_file)
     if mounted:
+        # In this case a utility like a configuration drive tool
+        # has *already* mounted the device we believe to be the
+        # configuration drive.
         _copy_config_from(mounted)
     else:
         with ironic_utils.mounted(vmedia_device_file) as vmedia_mount_point:
+            # In this case, we use a temporary folder and extract the contents
+            # for our configuration.
             _copy_config_from(vmedia_mount_point)
+    # As a last act, just make sure there is nothing else mounted.
+    _unmount_any_config_drives()
 
 
 def _get_cached_params():
@@ -622,7 +632,7 @@ def _parse_capabilities_str(cap_str):
     """Extract capabilities from string.
 
     :param cap_str: string meant to meet key1:value1,key2:value2 format
-    :return: a dictionnary
+    :return: a dictionary
     """
     LOG.debug("Parsing capability string %s", cap_str)
     capabilities = {}
@@ -689,7 +699,7 @@ def get_node_boot_mode(node):
     'instance_info/capabilities' of node. Otherwise it directly look for boot
     mode hints into
 
-    :param node: dictionnary.
+    :param node: dictionary.
     :returns: 'bios' or 'uefi'
     """
     instance_info = node.get('instance_info', {})
@@ -697,7 +707,7 @@ def get_node_boot_mode(node):
     node_caps = parse_capabilities(node.get('properties', {}))
 
     if _is_secure_boot(instance_info_caps, node_caps):
-        LOG.debug('Deploy boot mode is implicitely uefi for because secure '
+        LOG.debug('Deploy boot mode is implicitly uefi because secure '
                   'boot is activated.')
         return 'uefi'
 
@@ -770,7 +780,7 @@ def determine_time_method():
 
     :returns: "ntpdate" if ntpdate has been found, "chrony" if chrony
               was located, and None if neither are located. If both tools
-              are present, "chrony" will supercede "ntpdate".
+              are present, "chrony" will supersede "ntpdate".
     """
     try:
         execute('chronyd', '-h')
@@ -944,3 +954,32 @@ def find_in_lshw(lshw, by_id=None, by_class=None, recursive=False, **fields):
             yield child
         if recursive:
             yield from find_in_lshw(child, by_id, recursive=True, **fields)
+
+
+def _unmount_any_config_drives():
+    """Umount anything mounted to /mnt/config
+
+    As part of the configuration drive model, utilities like cloud-init
+    and glean leverage a folder at /mnt/config to convey configuration
+    to a booting OS.
+
+    The possibility exists that one of the utilities mounted one or multiple
+    such folders, even if the configuration was not used, and this can
+    result in locked devices which can prevent rebuild operations from
+    completing successfully as long as the folder is mounted, it is
+    a "locked" device to the operating system.
+    """
+    while os.path.ismount('/mnt/config'):
+        _early_log('Issuing an umount command for /mnt/config...')
+        execute('umount', '/mnt/config')
+        time.sleep(1)
+
+
+def is_char_device(path):
+    '''Check if the specified path is a character device.'''
+    try:
+        return stat.S_ISCHR(os.stat(path).st_mode)
+    except OSError:
+        # Likely because of insufficient permission,
+        # race conditions or I/O related errors.
+        return False
